#!/usr/bin/python

import math

class imageFeatures:

	def __init__(self, iMatrix):
		self.imageMatrix = iMatrix
		self.outline = {}
		self.trace = []
		self.q1 = {}
		self.q2 = {}
		self.q3 = {}
		self.q4 = {}
		self.left = {}
		self.right = {}
		self.roi = {}
		self.mean_q1 = ()
		self.mean_q2 = ()
		self.mean_q3 = ()
		self.mean_q4 = ()
		self.findOutline(self.imageMatrix, self.outline)
		self.midline = self.findMidline(self.imageMatrix, self.outline)
	
	def findOutline(self, imageMatrix, outline):
		for i in range(len(imageMatrix)): #iterates through rows
			first = ()
			ff = 0
			last = ()
			for j in range(len(imageMatrix[i])):
				if ff == 0 and imageMatrix[i][j] > 0:
					ff = 1
					first = (i,j)
				elif ff == 1 and imageMatrix[i][j] > 0:
					last = (i,j)				
			if len(first) > 0:		
				outline[first] = 0
			if len(last) > 0:
				outline[last] = 0			

		for j in range(len(imageMatrix[0])): #iterates through columns
			first = ()
			ff = 0
			last = ()
			for i in range(len(imageMatrix)):
				if ff == 0 and imageMatrix[i][j] > 0:
					ff = 1
					first = (i,j)
				elif ff == 1 and imageMatrix[i][j] > 0:
					last = (i,j)				
			if len(first) > 0:		
				outline[first] = 0
			if len(last) > 0:
				outline[last] = 0	

	def findMidline(self, imageMatrix, outline):
		oa = outline.keys()
		coldifs = []
		rowdifs = []
		midline = []
		for p1 in oa:
			for p2 in oa:
				rowdifs.append(math.fabs(p1[0] - p2[0]))
				coldifs.append(math.fabs(p1[1] - p2[1]))
		rowdif = sum(rowdifs)/len(rowdifs)
		coldif = sum(coldifs)/len(coldifs)
		first_half = []
		second_half = []

		if rowdif > coldif:
			#go through rows to find midline
			midline = []
			for i in range(len(imageMatrix)):
				row = []
				for j in range(len(imageMatrix[i])):
					if imageMatrix[i][j] > 0:
						row.append((i,j))
				if len(row) > 1:
					self.trace.append(len(row))
					midline.append((i,int((row[-1][1] + row[0][1])/2.0)))
					first_half.extend(row[0:len(row)/2])
					second_half.extend(row[len(row)/2:])
		else:
			#go through columns to find midline
			midline = []
			for j in range(len(imageMatrix[0])):
				col = []
				for i in range(len(imageMatrix)):
					if imageMatrix[i][j] > 0:
						col.append((i,j))
				if len(col) > 1:
					self.trace.append(len(col))
					midline.append((int((col[-1][0] + col[0][0])/2.0),j))
					first_half.extend(col[0:len(col)/2])
					second_half.extend(col[len(col)/2:])
		for point in first_half:
			self.left[point] = 0
			self.roi[point] = 0
		for point in second_half:
			self.right[point] = 0
			self.roi[point] = 0
		#print len(first_half)
		#print len(second_half)
		self.setQuadrants(first_half, second_half)
		return midline		
		
	def setQuadrants(self, first_half, second_half):
		tmp_quad = first_half[0:len(first_half)/2]
		mean_x = 0
		mean_y = 0
		for pix in tmp_quad:
			mean_x += pix[0]
			mean_y += pix[1]
			self.q1[pix] = 0
		self.mean_q1 = (mean_x/len(self.q1),mean_y/len(self.q1))
		
		mean_x = 0
		mean_y = 0
		tmp_quad = first_half[len(first_half)/2:]
		for pix in tmp_quad: 
			mean_x += pix[0]
			mean_y += pix[1]
			self.q2[pix] = 0
		self.mean_q2 = (mean_x/len(self.q2),mean_y/len(self.q2))
		
		mean_x = 0
		mean_y = 0
		tmp_quad = second_half[0:len(second_half)/2]
		for pix in tmp_quad:
			self.q3[pix] = 0
			mean_x += pix[0]
			mean_y += pix[1]
		self.mean_q3 = (mean_x/len(self.q3),mean_y/len(self.q3))
		
		mean_x = 0
		mean_y = 0
		tmp_quad = second_half[len(second_half)/2:]
		for pix in tmp_quad:
			self.q4[pix] = 0
			mean_x += pix[0]
			mean_y += pix[1]
		self.mean_q4 = (mean_x/len(self.q4), mean_y/len(self.q4))
		
				