from PIL import Image
import numpy as np
import scipy.ndimage as si
from scipy import spatial as ss
import argparse
import sys
import random
import os

import thresholding
from thresholding import *
import imageFeatures
from imageFeatures import *

def show_labeled_pix(im, flat_labels, keep_labels):
	im_show = im.convert("RGB")
	new_pix = im_show.getdata()
	pix = []
	for i in range(len(new_pix)):
		if flat_labels[i] in keep_labels:
			pix.append((255,0,0))
		else:
			pix.append((new_pix[i]))
	im_show.putdata(pix)
	im_show.show()

def show_quadrants(im, quadrants):
	im_show = im.convert("RGB")
	color_index = 0
	colors = [(255,0,0),(0,255,0),(0,0,255),(255,0,255)]
	for q in quadrants:
		for p in q:
			im_show.putpixel((p[1],p[0]),colors[color_index])
		color_index+=1	
	im_show.show()
	
def show_random_features(im, rfs):
	im_show = im.convert("RGB")
	for p in rfs:
		for i in range(2):
			for j in range(2):
				im_show.putpixel((p[1]+i,p[0]+j),(0,0,255))
	im_show.show()	

def averageDistance(trace_1, trace_2):
	distance = 0
	size = len(trace_1)
	if len(trace_1) < len(trace_2):
		while len(trace_1) < len(trace_2):
			index = random.randint(0, len(trace_2)-1)
			trace_2.pop(index)
	elif len(trace_1) > len(trace_2):
		size = len(trace_2)
		while len(trace_1) > len(trace_2):
			index = random.randint(0, len(trace_1)-1)
			trace_1.pop(index)
	
	for i in range(size):
		#compute the euclidean distance between graphs
		distance += (trace_1[i] - trace_2[i])**2.0
	distance = distance**(0.5)
	return distance

def more_extreme(r1,r2): #compare r1 to r2 to see if r2 is more extreme
	if r1 == r2:
		return False
	else:
		if r1 == 1.0:
			return True
		elif r1 < 1.0:
			if r2 < r1:
				return True
			else:
				return False
		else:
			if r2 > r1:
				return True
			else:
				return False
	print "problem, shouldn't have made it here"
					
def wiki_threshold(histogram, thresh_guess):
	threshold = 0
	index = 0
	pre_below = 0
	pre_above = 0
	while pre_below == 0 or pre_above == 0 and thresh_guess > 0:
		pre_below = sum(histogram[:thresh_guess])
		pre_above = sum(histogram[thresh_guess:])
		if pre_below == 0 or pre_above == 0:
			thresh_guess = thresh_guess - 1
	if thresh_guess < 2:
		print "thresholding can't find an appropriate starting value, check image"
		raise Exception
	while (threshold != thresh_guess and index < 50):
		index += 1
		sum_before = 0
		total_before = 0
		average_before = 0
		sum_after = 0
		total_after = 0
		average_after = 0
		#compute average_before
		for i in range(thresh_guess):
			sum_before += i*histogram[i]
			total_before += histogram[i]
		i = thresh_guess	
		while i < len(histogram):
			sum_after += i*histogram[i]
			total_after += histogram[i]
			i += 1
		average_before = sum_before / total_before
		average_after = sum_after / total_after
		threshold = thresh_guess
		thresh_guess = (average_before + average_after) / 2
	return threshold					

parser = argparse.ArgumentParser(description="Process folder of images")
parser.add_argument("image_folder",help="image folder path, assumes the images are 8 bit tifs; will not process the images otherwise",type=str)
parser.add_argument("output_file",help="output file name; outputs a csv file with processing information",type=str)
parser.add_argument("-cn","--count_neoblasts",help="Indicates whether to count neoblasts in the animal",action="store_true")
parser.add_argument("-nd", "--neoblast_distribution", help="Indicates whether to test for assymetry in neoblast distribution",action="store_true")
parser.add_argument("-morph","--morphologies",help="Generate distance matrix of images based on animal morphology",action="store_true")
parser.add_argument("-tm", "--threshold_method", help="Specifies the thresholding method, 1=niblak, 2=multiresultion",type=int,choices = [1,2], default=1)
parser.add_argument("-bs", "--box_size", help="Specifies the box size for niblack's method",type=int,default=10)
parser.add_argument("-slp", "--slope", help="Specifies the offset factor for niblack's method", type=float, default=2.3)
parser.add_argument("-mxfb","--max_fiber_size",help="specifies maximum fiber size (in pixels)",type=int,default=200)
parser.add_argument("-mnfb","--min_fiber_size",help="specifies minimum fiber size (in pixels)",type=int,default=10)
args = parser.parse_args()

if args.count_neoblasts == False and args.neoblast_distribution == False and args.morphologies == False:
	print "Please specifiy analysis to be done on the images, use option -h for help"
	print "Nothing to be done with the images, going to exit"
	sys.exit(0)

path = (args.image_folder.rstrip("/"))+"/"
images = os.listdir(path)
image_index = 0
morph_sizes_r = {}
morph_sizes_l = {}
out = open(args.output_file,'w')
out.write("image name,number of neoblasts,animal size (pixels),neoblast density (per pixel),a/p p-value,l/r p-value \n")
for image in images:
	image_index+=1
	print "Processing image " + str(image_index) + " of " + str(len(images))
	im = Image.open(path+image)
	if im.mode != "L":
		print "Image must be 8 bit, going to skip image"
		continue
	

	#isolate animal in image
	pix = np.array(im.getdata(), np.uint8).reshape((im.size[1],im.size[0]))
	threshold = wiki_threshold(im.histogram(), 40)
	isolated = pix*(pix >= threshold)
	labeled_array, num_features = si.label(isolated,structure=[[1,1,1],[1,1,1],[1,1,1]])
	label_hist = si.measurements.histogram(labeled_array,1,num_features,num_features)
	worm_size = max(label_hist)
	if num_features > 1:
		max_feature = np.argmax(label_hist)+1
		if max(label_hist)/float(len(pix)) > .05: #worm has to cover at least 5% of the image to count as being found
			isolated = isolated*(labeled_array == max_feature)
		else:
			print "Cannot isolate animal in image, most likely background flourescense in animal is not sufficient to isolate it in the image."
			print "The particles in the image will still be counted if -cn flag is on but any other analysis will not take place."
			worm_size = 0
			args.neoblast_distribution = False
			args.morphologies = False
	
	if args.count_neoblasts == True or args.neoblast_distribution == True:			
		#now threshold the animal to isolate the neoblasts
		thresh = thresholding(args.threshold_method, isolated.flatten(), im.size[1], im.size[0], args.box_size, args.slope)
		diff = isolated - thresh.thresh_surface
		neoblasts = isolated*(diff > 0)
		labeled_array, num_features = si.label(neoblasts,structure=[[1,1,1],[1,1,1],[1,1,1]])
		label_hist = si.measurements.histogram(labeled_array,1,num_features,num_features)
		condition = (label_hist > args.min_fiber_size) & (label_hist < args.max_fiber_size)
		fiber_sizes = label_hist[condition]
		fiber_labels = np.arange(1,num_features+1)[condition]
		keep_labels = dict.fromkeys(fiber_labels,0)
		neo_denisty = 0
		if worm_size > 0:
			neo_density = float(len(fiber_labels))/float(worm_size)
		if args.neoblast_distribution == False:
			out.write(image+","+str(len(fiber_labels))+","+str(worm_size)+","+str(neo_density)+",0,0 \n")
		else:
			out.write(image+","+str(len(fiber_labels))+","+str(worm_size)+","+str(neo_density)+",")
		
		#get outline and quadrants of animal, will also need the com of each point
	wm_features = 0
	if args.neoblast_distribution == True:
		wm_features = imageFeatures(isolated)
		#show_quadrants(im, [wm_features.roi])
		#calculate anterior / posterior and left / right ratios then compare for most extreme ratio
		coms = si.measurements.center_of_mass(labeled_array,labeled_array,fiber_labels)
		feature_qs = [0, 0, 0, 0]
		q_means = [wm_features.mean_q1,wm_features.mean_q2,wm_features.mean_q3,wm_features.mean_q4]
		for c in coms:
			min_index = np.argmin(ss.distance.cdist([c],q_means))
			feature_qs[min_index]+=1
		denominator = float(feature_qs[1]+feature_qs[3])
		if denominator == 0:
			denominator = 1.0	
		original_ap = float(feature_qs[0]+feature_qs[2])/float(denominator)
		denominator = float(feature_qs[2]+feature_qs[3])
		if denominator == 0:
			denominator = 1.0
		original_lr = float(feature_qs[0]+feature_qs[1])/float(denominator)
		#print "original_ap: " + str(original_ap)
		#print "original_lr: " + str(original_lr)
		#do 100 permutations randomly selecting points from the roi to represent the features and compute ratios
		num_features = len(coms)
		#print len(coms)
		extreme_ap = 0
		extreme_lr = 0
		permutations = 100
		for i in range(permutations):
			feature_qs = [0, 0, 0, 0]
			random_features = random.sample(wm_features.roi,num_features)
			#show_random_features(im, random_features)
			for p in random_features:
				min_index = np.argmin(ss.distance.cdist([p],q_means))
				feature_qs[min_index]+=1
			#print feature_qs
			denominator = float(feature_qs[1]+feature_qs[3])
			if denominator == 0:
				denominator = 1.0	
			tmp_ap = float(feature_qs[0]+feature_qs[2])/float(denominator)
			denominator = float(feature_qs[2]+feature_qs[3])
			if denominator == 0:
				denominator = 1.0
			tmp_lr = float(feature_qs[0]+feature_qs[1])/float(denominator)
			#print tmp_ap,tmp_lr
			if more_extreme(original_ap,tmp_ap) == True:
				extreme_ap += 1
			if more_extreme(original_lr,tmp_lr) == True:
				extreme_lr += 1
		out.write(str(extreme_ap/float(permutations))+","+str(extreme_lr/float(permutations))+"\n")
	if args.morphologies == True:
		if args.neoblast_distribution == False:
			wm_features = imageFeatures(isolated)
		av = np.mean(wm_features.trace)
		for i in range(len(wm_features.trace)):
			wm_features.trace[i] = float(wm_features.trace[i])/av

		morph_sizes_l[image] = wm_features.trace 
		cp_line_sizes = wm_features.trace[:]
		cp_line_sizes.reverse()
		morph_sizes_r[image] = cp_line_sizes
out.close()

if args.morphologies == True:
	#compute smallest average distances from each graph to another
	wells = morph_sizes_l.keys()
	wells.sort()
	distances = {}
	make_distances_equal = {}
	for well in wells:
		#print 'well: ',well
		distances[well] = []
		for well2 in wells:
			#print 'well2: ',well2
			#compute the euclidean distance between graphs
			min_dist = 99999999999999.99
			for j in [1,2,3,4]:
				tmp_dist = 0
				if j == 1:
					tmp_dist = averageDistance(morph_sizes_l[well], morph_sizes_l[well2])
				elif j == 2:
					tmp_dist = averageDistance(morph_sizes_l[well], morph_sizes_r[well2])
				elif j == 3:
					tmp_dist = averageDistance(morph_sizes_r[well], morph_sizes_l[well2])
				else:
					tmp_dist = averageDistance(morph_sizes_r[well], morph_sizes_r[well2])
				
				if tmp_dist < min_dist:
					min_dist = tmp_dist
				tmp_key1 = well+"-"+well2
				tmp_key2 = well2+"-"+well
				if tmp_key1 in make_distances_equal:
					min_dist = make_distances_equal[tmp_key1]
				elif tmp_key2 in make_distances_equal:
					min_dist = make_distances_equal[tmp_key2]
				else:
					make_distances_equal[tmp_key1] = min_dist
			distances[well].append(min_dist)
	
	f = open('distance_matrix.txt', 'w')
	wells = distances.keys()
	wells.sort()
	for well in wells:
		f.write(well+",")
	f.write("\n")
	
	for i in range(len(distances[wells[0]])):
		for well in wells:
			f.write(str(distances[well][i])+",")
		f.write("\n")
	f.close()
				
		
	

	
	

