from PIL import Image
import numpy as np
import scipy.ndimage as si
from scipy import spatial as ss
import argparse
import sys
import random

import thresholding
from thresholding import *
import imageFeatures
from imageFeatures import *

def show_labeled_pix(im, flat_labels, keep_labels):
	im_show = im.convert("RGB")
	new_pix = im_show.getdata()
	pix = []
	for i in range(len(new_pix)):
		if flat_labels[i] in keep_labels:
			pix.append((255,0,0))
		else:
			pix.append((new_pix[i]))
	im_show.putdata(pix)
	im_show.show()

def show_quadrants(im, quadrants):
	im_show = im.convert("RGB")
	color_index = 0
	colors = [(255,0,0),(0,255,0),(0,0,255),(255,0,255)]
	for q in quadrants:
		for p in q:
			im_show.putpixel((p[1],p[0]),colors[color_index])
		color_index+=1	
	im_show.show()
	
def show_random_features(im, rfs):
	im_show = im.convert("RGB")
	for p in rfs:
		for i in range(2):
			for j in range(2):
				im_show.putpixel((p[1]+i,p[0]+j),(0,0,255))
	im_show.show()	

def more_extreme(r1,r2): #compare r1 to r2 to see if r2 is more extreme
	if r1 == r2:
		return False
	else:
		if r1 == 1.0:
			return True
		elif r1 < 1.0:
			if r2 < r1:
				return True
			else:
				return False
		else:
			if r2 > r1:
				return True
			else:
				return False
	print "problem, shouldn't have made it here"
					
def wiki_threshold(histogram, thresh_guess):
	threshold = 0
	index = 0
	pre_below = 0
	pre_above = 0
	while pre_below == 0 or pre_above == 0 and thresh_guess > 0:
		pre_below = sum(histogram[:thresh_guess])
		pre_above = sum(histogram[thresh_guess:])
		if pre_below == 0 or pre_above == 0:
			thresh_guess = thresh_guess - 1
	if thresh_guess < 2:
		print "thresholding can't find an appropriate starting value, check image"
		raise Exception
	while (threshold != thresh_guess and index < 50):
		index += 1
		sum_before = 0
		total_before = 0
		average_before = 0
		sum_after = 0
		total_after = 0
		average_after = 0
		#compute average_before
		for i in range(thresh_guess):
			sum_before += i*histogram[i]
			total_before += histogram[i]
		i = thresh_guess	
		while i < len(histogram):
			sum_after += i*histogram[i]
			total_after += histogram[i]
			i += 1
		average_before = sum_before / total_before
		average_after = sum_after / total_after
		threshold = thresh_guess
		thresh_guess = (average_before + average_after) / 2
	return threshold					

parser = argparse.ArgumentParser(description="Process single image")
parser.add_argument("image_name",help="image file name and path, image must be 8 bit")
parser.add_argument("-cn","--count_neoblasts",help="Indicates whether to count neoblasts in an animal",action="store_true")
parser.add_argument("-nd", "--neoblast_distribution", help="Indicates whether to test for assymetry in neoblast distribution",action="store_true")
parser.add_argument("-tm", "--threshold_method", help="Specifies the thresholding method, 1=niblak, 2=multiresultion",type=int,choices = [1,2], default=1)
parser.add_argument("-bs", "--box_size", help="Specifies the box size for niblack's method",type=int,default=10)
parser.add_argument("-slp", "--slope", help="Specifies the offset factor for niblack's method", type=float, default=2.3)
parser.add_argument("-mxfb","--max_fiber_size",help="specifies maximum fiber size (in pixels)",type=int,default=200)
parser.add_argument("-mnfb","--min_fiber_size",help="specifies minimum fiber size (in pixels)",type=int,default=10)
args = parser.parse_args()

if args.count_neoblasts == False and args.neoblast_distribution == False:
	print "Please specifiy analysis to be done on the image, use option -h for help"
	print "Nothing to be done with the image, going to exit"
	sys.exit(0)
	
im = Image.open(args.image_name)
if im.mode != "L":
	print "Image must be 8 bit, going to exit"
	sys.exit(0)

if args.count_neoblasts or args.neoblast_distribution:
	#isolate animal in image
	pix = np.array(im.getdata(), np.uint8).reshape((im.size[1],im.size[0]))
	threshold = wiki_threshold(im.histogram(), 40)
	isolated = pix*(pix >= threshold)
	labeled_array, num_features = si.label(isolated,structure=[[1,1,1],[1,1,1],[1,1,1]])
	label_hist = si.measurements.histogram(labeled_array,1,num_features,num_features)
	worm_size = max(label_hist)
	if num_features > 1:
		max_feature = np.argmax(label_hist)+1
		if max(label_hist)/float(len(pix)) > .05: #worm has to cover at least 5% of the image to count as being found
			isolated = isolated*(labeled_array == max_feature)
		else:
			print "Cannot isolate animal in image, most likely background flourescense in animal is not sufficient to isolate it in the image."
			print "The particles in the image will still be counted but any other analysis will not take place."
			worm_size = 0
			args.neoblast_distribution = False
				
	#now threshold the animal to isolate the neoblasts
	thresh = thresholding(args.threshold_method, isolated.flatten(), im.size[1], im.size[0], args.box_size, args.slope)
	diff = isolated - thresh.thresh_surface
	neoblasts = isolated*(diff > 0)
	labeled_array, num_features = si.label(neoblasts,structure=[[1,1,1],[1,1,1],[1,1,1]])
	label_hist = si.measurements.histogram(labeled_array,1,num_features,num_features)
	condition = (label_hist > args.min_fiber_size) & (label_hist < args.max_fiber_size)
	fiber_sizes = label_hist[condition]
	fiber_labels = np.arange(1,num_features+1)[condition]
	keep_labels = dict.fromkeys(fiber_labels,0)
	print "number of neoblasts counted: " + str(len(fiber_labels))
	if worm_size > 0:
		print "size of animal (pixels): " + str(worm_size)
		print "neoblast density in animal (per pixel): " + str(len(fiber_labels)/float(worm_size))
	show_labeled_pix(im, labeled_array.flatten(), keep_labels)	
	
	#get outline and quadrants of animal, will also need the com of each point
	if args.neoblast_distribution == True:
		wm_features = imageFeatures(isolated)
		#show_quadrants(im, [wm_features.roi])
		#calculate anterior / posterior and left / right ratios then compare for most extreme ratio
		coms = si.measurements.center_of_mass(labeled_array,labeled_array,fiber_labels)
		feature_qs = [0, 0, 0, 0]
		q_means = [wm_features.mean_q1,wm_features.mean_q2,wm_features.mean_q3,wm_features.mean_q4]
		for c in coms:
			min_index = np.argmin(ss.distance.cdist([c],q_means))
			feature_qs[min_index]+=1
		denominator = float(feature_qs[1]+feature_qs[3])
		if denominator == 0:
			denominator = 1.0	
		original_ap = float(feature_qs[0]+feature_qs[2])/float(denominator)
		denominator = float(feature_qs[2]+feature_qs[3])
		if denominator == 0:
			denominator = 1.0
		original_lr = float(feature_qs[0]+feature_qs[1])/float(denominator)
		#print "original_ap: " + str(original_ap)
		#print "original_lr: " + str(original_lr)
		#do 100 permutations randomly selecting points from the roi to represent the features and compute ratios
		num_features = len(coms)
		#print len(coms)
		extreme_ap = 0
		extreme_lr = 0
		permutations = 100
		for i in range(permutations):
			feature_qs = [0, 0, 0, 0]
			random_features = random.sample(wm_features.roi,num_features)
			#show_random_features(im, random_features)
			for p in random_features:
				min_index = np.argmin(ss.distance.cdist([p],q_means))
				feature_qs[min_index]+=1
			#print feature_qs
			denominator = float(feature_qs[1]+feature_qs[3])
			if denominator == 0:
				denominator = 1.0	
			tmp_ap = float(feature_qs[0]+feature_qs[2])/float(denominator)
			denominator = float(feature_qs[2]+feature_qs[3])
			if denominator == 0:
				denominator = 1.0
			tmp_lr = float(feature_qs[0]+feature_qs[1])/float(denominator)
			#print tmp_ap,tmp_lr
			if more_extreme(original_ap,tmp_ap) == True:
				extreme_ap += 1
			if more_extreme(original_lr,tmp_lr) == True:
				extreme_lr += 1
		print "p-value for anterior posterior asymmetries: " + str(extreme_ap/float(permutations))
		print "p-value for left right asymmetries: " + str(extreme_lr/float(permutations))
		
		
	

	
	

