from PIL import Image
import os
import numpy as np
import math
import scipy.ndimage as nd

class thresholding:

	def __init__(self, method, pixels, rows, cols, box_side, slope):
		#pixels must be a flattened array such that PIL would recognize it
		if method == 1:
			self.thresh_surface = self.find_niblack_surface(pixels,rows,cols,box_side,slope)
		else:
			self.thresh_surface = self.find_multiresolution_surface(pixels,rows,cols)
		
	def find_niblack_surface(self,pixels,rows,cols,box_side,slope): 
		#write data for c implementation of niblack
		out = open("imagedata.txt","w")
		for p in pixels:
			out.write(str(p)+"\n")
		out.close()
		#call c program, the arguments will be row, col, box_size, slope
		cargs = [rows,cols,box_side,slope]
		#print cargs
		os.system("./a.out "+" ".join(map(str,cargs)))
		#bring in surface data
		thresh_surface = []
		f = open("surface.txt","r")
		for line in f:
			if len(line.strip()) > 0:
				thresh_surface.append(int(float(line.strip())))
		f.close()
		return np.array(thresh_surface).reshape(rows,cols)
	
	def find_multiresolution_surface(self,pixels,rows,cols):
		im = Image.new("L",(cols,rows))
		im.putdata(pixels)
		side = rows
		if cols < rows:
			side = cols
		im_sq = im.resize((side,side))
		
		#important note: im.size gives (width, height), while numpy looks at multidimensional arrays as [height, width]
		zvals = np.array(im_sq.getdata()).reshape(im_sq.size[1],im_sq.size[0])
		zgrad = np.array(np.gradient(zvals))
		#convert gradient values to norm of x,y derivatives
		znorm = []
		for i in range(im_sq.size[1]):
			znorm.append([])
			for j in range(im_sq.size[0]):
				znorm[i].append(float(zgrad[0][i][j]**2.0 + zgrad[1][i][j]**2.0)**0.5)
		flat_norms = list(np.reshape(np.array(znorm),im_sq.size[1]*im_sq.size[0]))
		flat_norms.sort(reverse=True)
		#choose 1500 points to be the supports
		support_cutoff = flat_norms[1500]
		support_points = self.get_support_points(znorm,support_cutoff,zvals,im_sq) #for gradient based method
		#actually create thresholding surface
		surface = np.zeros((im_sq.size[1],im_sq.size[0]))
		num_levels = int(math.log(side, 2))
		levels = []
		for i in range(num_levels+1):
			#print "resolution level: " + str(i)
			inc = side/(2**i)
			#print inc
			#for each box, calculate the coefficient and then update
			#the surface and support points concurrently
			#in loops make indices inclusive
			col = 0
			while col < side:
				row = 0
				while row < side:
					coeff = 0.0
					num = 0
					for i in range(col,col+inc):
						for j in range(row, row+inc):
							if (i,j) in support_points:
								coeff += support_points[(i,j)]
								num += 1
					if num != 0:
						coeff = float(coeff)/num
						for i in range(col,col+inc):
							for j in range(row, row+inc):
								if i < side and j < side:
									if (i,j) in support_points:
										support_points[(i,j)] = support_points[(i,j)] - coeff
									surface[i][j] += coeff
					row += inc
				col += inc
				
		znew = nd.gaussian_filter(surface, sigma=3.0, order=0)
		im_surface = Image.new("L",(side,side))
		im_surface.putdata(znew.flatten())
		#im_surface.show()
		orig_dims_surface = np.array(im_surface.resize((cols,rows)).getdata()).reshape((rows,cols))
		return orig_dims_surface
		
		
	def update(self,level_num, side):
		inc = side/(2**level_num)
		#print inc
		#for each box, calculate the coefficient and then update
		#the surface and support points concurrently
		#in loops make indices inclusive
		col = 0
		while col < side:
			row = 0
			while row < side:
				coeff = 0.0
				num = 0
				for i in range(col,col+inc):
					for j in range(row, row+inc):
						if (i,j) in self.support_points:
							coeff += self.support_points[(i,j)]
							num += 1
				if num != 0:
					coeff = float(coeff)/num
					for i in range(col,col+inc):
						for j in range(row, row+inc):
							if i < side and j < side:
								if (i,j) in self.support_points:
									self.support_points[(i,j)] = self.support_points[(i,j)] - coeff
								self.surface[i][j] += coeff
				row += inc
			col += inc		
			
			
	def get_support_points(self,znorm,sc,zvals,im):
		sp  ={}
		for i in range(im.size[1]):
			for j in range(im.size[0]):
				if znorm[i][j] >= sc:
					sp[(i,j)] = zvals[i][j]
		return sp